{% extends edc_base_template %}

{% load static %}

{% block main %}
<div class="container">

    <div id="div-home-right" class="col-md-8">
         <div class="panel panel-default">
    <div class="panel-heading">Consent Versions</div>
    <table class="table table-hover responsive">
    	<thead>
      		<tr>
       			<th>Model</th>
        		<th>Version</th>
        		<th>Start Date</th>
        		<th>End Date</th>
        		<th>Updates</th>
     	 	</tr>
    	</thead>
    	<tbody>
      	{% for consent in  consents %}
     	 <tr>
        	<td>{{consent.verbose_name|title}}</td>
        	<td>{{consent.version}}</td>
          	<td>{{consent.start_datetime|date:"Y-M-d"}}</td>
        	<td>{{consent.start_datetime|date:"Y-M-d"}}</td>
        	<td>{{consent.updates_versions}}</td>
      	</tr>
      	{% endfor %}
    	</tbody>
 	 </table>
        </div>
     <div class="panel panel-default">
        <div class="panel-heading">Consent Verification</div>
    </div>
    </div>
    <div id="div-home-right" class="col-md-4">
        <div class="panel panel-default">
        <div class="panel-heading">Administration</div>
        <ul class="nav nav-pills nav-stacked">
        <li><a href="{% url 'edc_consent:edc_consent_admin:index' %}">Admin Site</a></li>
        {% for consent in consents %}
        <li><a href="">{{ consent.name }}</a></li>
        {% endfor %}
        </ul>
        </div>
    </div>

</div>
{% endblock main %}